//Copyright (c) 1999, Art Gittleman
//This example is provided WITHOUT ANY WARRANTY either expressed or implied.

/* Uses wait and notify to enable
 * producer and consumer threads to
 * cooperate in using a buffer.
 */

// Updated to concurrent by Dr. Spiegel
// Updated to use version 5 Synchronous queue to do rendezvous
//   Buffer is now an ArrayBlockingQueue 

import java.util.concurrent.*;
  
public class ConcurrentPutGet {

  // Buffer is a synchronized array
  static ArrayBlockingQueue<Integer> Buffer;

  class Producer extends Thread {

     public void run() {
       for(int i=1; ; i++)
         try {
             Buffer.put(i);
	     System.out.println("Put "+i);
	     sleep(100);
           }catch (InterruptedException e){
              e.printStackTrace();
           }
     }
  }

  class Consumer extends Thread {

     public void run() {
      while (true)
         try {
           int i=Buffer.take();
	   System.out.println("Took "+i);
	   sleep(100);
         }catch (InterruptedException e) {
               e.printStackTrace();
         }
     }
  }

  public static void main(String[] args) {
    if (args.length!=3) {
	System.out.println("Usage: PutGet  <Buffer Size> <#Producers>  <#Consumers>");
        System.exit(0);
    }
    ConcurrentPutGet pg = new ConcurrentPutGet();
    Buffer=new ArrayBlockingQueue(Integer.parseInt(args[0]),true);
    Producer p[] = new Producer[Integer.parseInt(args[1])];
    for (int pIdx=0;pIdx<Integer.parseInt(args[1]);pIdx++) {
      p[pIdx]=pg.new Producer();
      p[pIdx].start();
    }
    Consumer c[] = new Consumer[Integer.parseInt(args[2])];
    for (int cIdx=0;cIdx<Integer.parseInt(args[2]);cIdx++) {
      c[cIdx]=pg.new Consumer();
      c[cIdx].start();
    }
  }
}

